@@ -1,4 +1,4 @@
-# Copyright 2016-2022 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2023 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -523,16 +523,34 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
 
     #: Number of tasks required by this test.
     #:
-    #: If the number of tasks is set to a number ``<=0``, ReFrame will try to
-    #: flexibly allocate the number of tasks, based on the command line option
-    #: |--flex-alloc-nodes|_. A negative number is used to indicate the minimum
-    #: number of tasks required for the test. In this case the minimum number
-    #: of tasks is the absolute value of the number, while Setting
-    #: :attr:`num_tasks` to ``0`` is equivalent to setting it to
+    #: If the number of tasks is set to zero or a negative value, ReFrame will
+    #: try to flexibly allocate the number of tasks based on the command line
+    #: option :option:`--flex-alloc-nodes`. A negative number is used to
+    #: indicate the minimum number of tasks required for the test. In this case
+    #: the minimum number of tasks is the absolute value of the number, while
+    #: Setting :attr:`num_tasks` to zero is equivalent to setting it to
     #: :attr:`-num_tasks_per_node
     #: <reframe.core.pipeline.RegressionTest.num_tasks_per_node>`.
     #:
-    #: :type: integral
+    #: Setting :attr:`num_tasks` to :obj:`None` has a scheduler-specific
+    #: interpretation, but in principle, passes the responsibility of producing
+    #: a correct job script to the user by setting the appropriate scheduler
+    #: options. More specifically, the different backends interpret the
+    #: :obj:`None` :attr:`num_tasks` as follows:
+    #:
+    #: - ``flux``: not applicable.
+    #: - ``local``: not applicable.
+    #: - ``lsf``: Neither the ``-nnodes`` nor the ``-n`` will be emitted.
+    #: - ``oar``: Resets it to ``1``.
+    #: - ``pbs``: Resets it to ``1``.
+    #: - ``sge``: not applicable.
+    #: - ``slurm``: Neither the ``--ntasks`` nor the ``--nodes`` option (if the
+    #:   :attr:`~config.systems.partitions.sched_options.use_nodes_option` is
+    #:   specified) will be emitted.
+    #: - ``squeue``: See ``slurm`` backend.
+    #: - ``torque``: See ``pbs`` backend.
+    #:
+    #: :type: integral or :obj:`None`
     #: :default: ``1``
     #:
     #: .. note::
@@ -542,10 +560,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #:     .. versionchanged:: 2.16
     #:        Negative :attr:`num_tasks` is allowed for specifying the minimum
     #:        number of required tasks by the test.
-    #:
-    #: .. |--flex-alloc-nodes| replace:: :attr:`--flex-alloc-nodes`
-    #: .. _--flex-alloc-nodes: manpage.html#cmdoption-flex-alloc-nodes
-    num_tasks = variable(int, value=1, loggable=True)
+    #:     .. versionchanged:: 4.1
+    #:        Allow :attr:`num_tasks` to be :obj:`None`.
+    num_tasks = variable(int, type(None), value=1, loggable=True)
 
     #: Number of tasks per node required by this test.
     #:
@@ -946,6 +963,11 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
     #: :default: :class:`True`
     build_locally = variable(typ.Bool, value=True, loggable=True)
 
+    # Special variables
+
+    #: Dry-run mode
+    _rfm_dry_run = variable(typ.Bool, value=False)
+
     def __new__(cls, *args, **kwargs):
         obj = super().__new__(cls)
 
@@ -984,9 +1006,8 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         self.__deferred_rfm_init.evaluate()
 
         # Build the default performance dict
-        if not self.perf_variables:
-            for fn in self._rfm_perf_fns.values():
-                self.perf_variables[fn._rfm_perf_key] = fn(self)
+        for fn in self._rfm_perf_fns.values():
+            self.perf_variables[fn._rfm_perf_key] = fn(self)
 
     def __init__(self):
         pass
@@ -1498,6 +1519,14 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         '''Check if the test is a fixture.'''
         return getattr(self, '_rfm_is_fixture', False)
 
+    def is_dry_run(self):
+        '''Check if the test runs in dry-run mode.
+
+        .. versionadded:: 4.1
+
+        '''
+        return self._rfm_dry_run
+
     def _resolve_fixtures(self):
         '''Resolve the fixture dependencies and inject the fixture handle.
 
@@ -1798,7 +1827,8 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             except OSError as e:
                 raise PipelineError('failed to prepare build job') from e
 
-            self._build_job.submit()
+            if not self.is_dry_run():
+                self._build_job.submit()
 
     @final
     def compile_wait(self):
@@ -1820,6 +1850,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
               more details.
 
         '''
+        if self.is_dry_run():
+            return
+
         self._build_job.wait()
 
         # We raise a BuildError when we an exit code and it is non zero
@@ -1939,9 +1972,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
             except OSError as e:
                 raise PipelineError('failed to prepare run job') from e
 
-            self._job.submit()
-
-        self.logger.debug(f'Spawned run job (id={self.job.jobid})')
+            if not self.is_dry_run():
+                self._job.submit()
+                self.logger.debug(f'Spawned run job (id={self.job.jobid})')
 
         # Update num_tasks if test is flexible
         if self.job.sched_flex_alloc_nodes:
@@ -1966,7 +1999,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         :raises reframe.core.exceptions.ReframeError: In case of errors.
 
         '''
-        if not self._build_job:
+        if not self._build_job or self.is_dry_run():
             return True
 
         return self._build_job.finished()
@@ -1995,7 +2028,7 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
               more details.
 
         '''
-        if not self._job:
+        if not self._job or self.is_dry_run():
             return True
 
         return self._job.finished()
@@ -2018,6 +2051,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
               more details.
 
         '''
+        if self.is_dry_run():
+            return
+
         self._job.wait()
 
     @final
@@ -2079,6 +2115,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
         elif not hasattr(self, 'sanity_patterns'):
             raise SanityError('sanity_patterns not set')
 
+        if self.is_dry_run():
+            return
+
         with osext.change_dir(self._stagedir):
             success = sn.evaluate(self.sanity_patterns)
             if not success:
@@ -2135,6 +2174,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
                 self.perf_variables[var] = sn.make_performance_function(expr,
                                                                         unit)
 
+        if self.is_dry_run():
+            return
+
         # Evaluate the performance function and retrieve the metrics
         with osext.change_dir(self._stagedir):
             for tag, expr in self.perf_variables.items():
@@ -2256,6 +2298,9 @@ class RegressionTest(RegressionMixin, jsonext.JSONSerializable):
               more details.
 
         '''
+        if self.is_dry_run():
+            return
+
         aliased = os.path.samefile(self._stagedir, self._outputdir)
         if aliased:
             self.logger.debug(
