@@ -1,4 +1,4 @@
-# Copyright 2016-2022 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
+# Copyright 2016-2023 Swiss National Supercomputing Centre (CSCS/ETH Zurich)
 # ReFrame Project Developers. See the top-level LICENSE file for details.
 #
 # SPDX-License-Identifier: BSD-3-Clause
@@ -45,7 +45,7 @@ class JobSchedulerMeta(abc.ABCMeta):
         obj = cls.__new__(cls, *args, **kwargs)
         if part_name:
             obj._config_prefix = (
-                f'systems/0/paritions/@{part_name}/sched_options'
+                f'systems/0/partitions/@{part_name}/sched_options'
             )
         else:
             obj._config_prefix = 'systems/0/sched_options'
@@ -174,7 +174,10 @@ class Job(jsonext.JSONSerializable, metaclass=JobMeta):
     #:    based on the test information.
     #:
     #: .. versionadded:: 3.11.0
-    num_tasks = variable(int, value=1)
+    #:
+    #: .. versionchanged:: 4.1
+    #:    Allow :obj:`None` values
+    num_tasks = variable(int, type(None), value=1)
 
     #: Number of tasks per node for this job.
     #:
@@ -487,7 +490,7 @@ class Job(jsonext.JSONSerializable, metaclass=JobMeta):
         This attribute might be useful in a flexible regression test for
         determining the actual nodes that were assigned to the test.
         For more information on flexible node allocation, see the
-        |--flex-alloc-nodes|_ command-line option
+        :option:`--flex-alloc-nodes` command-line option.
 
         This attribute is *not* supported by the ``pbs`` scheduler backend.
 
@@ -514,7 +517,7 @@ class Job(jsonext.JSONSerializable, metaclass=JobMeta):
 
     def prepare(self, commands, environs=None, prepare_cmds=None, **gen_opts):
         environs = environs or []
-        if self.num_tasks <= 0:
+        if self.num_tasks is not None and self.num_tasks <= 0:
             getlogger().debug(f'[F] Flexible node allocation requested')
             num_tasks_per_node = self.num_tasks_per_node or 1
             min_num_tasks = (-self.num_tasks if self.num_tasks else
